#include "stdafx.h"
#include "rect.h"

#include "style.h"

namespace GDIView
{

void Rect::gradient(Gdiplus::Graphics& someGraphics, const Style& aStyle, const int aX, const int aY, const int aWidth, const int aHeight)
{
	someGraphics.FillRectangle(&Gdiplus::LinearGradientBrush(Gdiplus::PointF(0.f, (float)aY),
															Gdiplus::PointF(0.f, (float)(aY + aHeight)),
															aStyle.myIn, aStyle.myOut),
															aX, aY, aWidth, aHeight);
}

const bool Rect::inside(const POINT& aPoint, const int anOriginX, const int anOriginY, const int aX, const int aY, const int aWidth, const int aHeight)
{
	const int CPX(aPoint.x - anOriginX), CPY(aPoint.y - anOriginY);

	return CPX > aX && CPY > aY && CPX < (aX + aWidth) && CPY < (aY + aHeight);
}

Rect::Rect()
:myStyle(NULL)
,myInside(false)
{
}

void Rect::set(const bool aDown, const Style& aStyle, const int aX, const int aY, const int aWidth, const int aHeight)
{
	myInside = false;
	myDown = aDown;
	myStyle = &aStyle;
	myX.set(aX);
	myY.set(aY);
	myWidth.set(aWidth);
	myHeight.set(aHeight);
}

void Rect::render(Gdiplus::Graphics& aGraphics, const int anOriginX, const int anOriginY)
{
	myX.lerp();
	myY.lerp();
	myWidth.lerp();
	myHeight.lerp();

	const int X(myX.get()),
				Y(myY.get()),
				WIDTH(myWidth.get()),
				HEIGHT(myHeight.get());

	if(myInside)
	{
		if(myDown)
		{
			aGraphics.FillRectangle(&Gdiplus::SolidBrush(myStyle->myOut), X + anOriginX, Y + anOriginY, WIDTH - 1, HEIGHT - 1);
		}
		else
		{
			aGraphics.FillRectangle(&Gdiplus::SolidBrush(myStyle->myIn), X + anOriginX, Y + anOriginY, WIDTH - 1, HEIGHT - 1);
		}
	}
	else
	{
		aGraphics.FillRectangle(&Gdiplus::LinearGradientBrush(Gdiplus::PointF(0.f, (float)(Y + anOriginY)),
															Gdiplus::PointF(0.f, (float)(Y + anOriginY + HEIGHT - 1)),
															myStyle->myIn, myStyle->myOut),
															X + anOriginX, Y + anOriginY, WIDTH - 1, HEIGHT - 1);
	}

	aGraphics.DrawRectangle(&Gdiplus::Pen(myStyle->myEdge), X + anOriginX, Y + anOriginY, WIDTH - 1, HEIGHT - 1);
}

const bool Rect::inside(const POINT& aPoint, const int anOriginX, const int anOriginY, const bool aSet)
{
	if(inside(aPoint, anOriginX, anOriginY, myX.get(), myY.get(), myWidth.get(), myHeight.get()))
	{
		myInside = aSet;
		return true;
	}
	else
	{
		return false;
	}
}

}