#include "stdafx.h"
#include "state.h"

namespace GDIView
{

State::State(const int aButtonWidth, const int aButtonHeight)
:myButtonWidth(aButtonWidth)
,myButtonHeight(aButtonHeight)
,myDrag(NULL)
,myDrop(NULL)
,myScroll(NULL)
,myWindow(0)
,myInsideRect(false)
,myScrollPos(-1)
{
	myDragPos.x = -1;
	myDragPos.y = -1;
}

void State::beginDrag(const int aScreenX, const int aScreenY, const Style& aStyle, const int aWidth, const int aHeight, const WCHAR* aText)
{
	const POINT CP(cursorPos());

	myDragPos.x = CP.x - aScreenX;
	myDragPos.y = CP.y - aScreenY;

	doDrag(aStyle, aWidth, aHeight, aText);
}

void State::doDrag(const Style& aStyle, const int aWidth, const int aHeight, const WCHAR* aText)
{
	const POINT CP(cursorPos());
	const int X(CP.x - myDragPos.x), Y(CP.y - myDragPos.y);

	if(aText)
		myDragText.set(X, Y, aText, 0xff000000);

	myDragRect.set(true, aStyle, X, Y, aWidth, aHeight);
}

void State::update()
{
	unsigned int i;

	for(i = 0; i < NUM_KEYSTATES; i++)
		myKeyStates[i].sample(i);

	if(!isDown(VK_LBUTTON))
	{
		myDrop = myDrag;
		myDrag = NULL;
		myDragPos.x = -1;
		myDragPos.y = -1;

		myScroll = NULL;
		myScrollPos = -1;
	}

	myInsideRect = false;
}

void State::render(Gdiplus::Graphics& someGraphics)
{
	if(myDrag)
	{
		myDragRect.render(someGraphics, 0, 0);
		myDragText.render(someGraphics, 0, 0);
	}
}

const float State::scrollX()
{
	float offset(0.f);
	POINT cp(cursorPos());

	if(myScrollPos != -1)
		offset = (float)(cp.x - myScrollPos);
	myScrollPos = cp.x;

	return offset;
}

const float State::scrollY()
{
	float offset(0.f);
	POINT cp(cursorPos());

	if(myScrollPos != -1)
		offset = (float)(cp.y - myScrollPos);
	myScrollPos = cp.y;

	return offset;
}

const bool State::isDown(const int aKey) const
{
	if(aKey >= 0 && aKey < NUM_KEYSTATES)
		return myKeyStates[aKey].isDown();
	else
		return false;
}

const bool State::wasPressed(const int aKey) const
{
	if(aKey >= 0 && aKey < NUM_KEYSTATES)
		return myKeyStates[aKey].wasPressed();
	else
		return false;
}

const POINT State::cursorPos() const
{
	POINT cp;

	GetCursorPos(&cp);
	ScreenToClient(myWindow, &cp);

	return cp;
}

const POINT State::dragPos() const
{
	POINT cp(cursorPos());

	cp.x -= myDragPos.x;
	cp.y -= myDragPos.y;

	return cp;
}

const void* State::dropOutside(const int aX, const int aY, const int aWidth, const int aHeight) const
{
	if(myDrop != NULL && !myInsideRect && Rect::inside(cursorPos(), aX, aY, 0, 0, aWidth, aHeight))
		return myDrop;
	else
		return NULL;
}

}