#include "stdafx.h"
#include "view.h"

#include "canvas.h"

namespace GDIView
{

#define WINDOW_CLASS L"GDIView"
static View* locView(NULL);
static ULONG_PTR locToken(0);

//local methods
LRESULT CALLBACK wndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC          hdc;
	PAINTSTRUCT  ps;

	switch(message)
	{
	case WM_PAINT:
		{
			hdc = BeginPaint(hWnd, &ps);
			Gdiplus::Graphics g(hdc);
			locView->render(g);	
			EndPaint(hWnd, &ps);
			return 0;
		}
		break;

	case WM_DESTROY:
		{
			PostQuitMessage(0);
			return 0;
		}
		break;

	default:
		{
			return DefWindowProc(hWnd, message, wParam, lParam);
		}
		break;
	}
}

View::View(const int aButtonWidth, const int aButtonHeight)
:myState(aButtonWidth, aButtonHeight)
{
	locView = this;
}

void View::init(HINSTANCE hInstance, INT iCmdShow, const int aWidth, const int aHeight, const WCHAR* aTitle)
{
	WNDCLASS c;
	Gdiplus::GdiplusStartupInput i;

	// Initialize GDI+.
	GdiplusStartup(&locToken, &i, NULL);

	c.style          = CS_HREDRAW | CS_VREDRAW;
	c.lpfnWndProc    = wndProc;
	c.cbClsExtra     = 0;
	c.cbWndExtra     = 0;
	c.hInstance      = hInstance;
	c.hIcon          = LoadIcon(NULL, IDI_APPLICATION);
	c.hCursor        = LoadCursor(NULL, IDC_ARROW);
	c.hbrBackground  = (HBRUSH)GetStockObject(WHITE_BRUSH);
	c.lpszMenuName   = NULL;
	c.lpszClassName  = WINDOW_CLASS;

	RegisterClass(&c);

	myState.myWindow = CreateWindow(
		WINDOW_CLASS,   // window class name
		aTitle,  // window caption
		WS_OVERLAPPEDWINDOW,      // window style
		CW_USEDEFAULT,            // initial x position
		CW_USEDEFAULT,            // initial y position
		aWidth,            // initial x size
		aHeight,            // initial y size
		NULL,                     // parent window handle
		NULL,                     // window menu handle
		hInstance,                // program instance handle
		NULL);                    // creation parameters

	ShowWindow(myState.myWindow, iCmdShow);
	UpdateWindow(myState.myWindow);
}

void View::cleanup()
{
	Gdiplus::GdiplusShutdown(locToken);
}

Window* View::window(const WCHAR* aTitle, const int aX, const int aY, const int aWidth, const int aHeight)
{
	return myWindows.window(myState, aTitle, aX, aY, aWidth, aHeight);
}

void View::draw()
{
	InvalidateRect(myState.myWindow, NULL, false);
}

void View::render(Gdiplus::Graphics& aGraphics)
{
	//canvas size of window
	RECT rect;
	GetClientRect(myState.myWindow, &rect);
	Canvas c(rect.right, rect.bottom, 0xff7f7f7f);

	//windows
	myWindows.render(myState, c.getGraphics());

	//render common state (drag and drop)
	myState.render(c.getGraphics());
	
	//present
	c.render(aGraphics);

	//sample new input
	myState.update();
}

}