#pragma once

#include "text.h"
#include "rect.h"

namespace GDIView
{

class Windows;
class State;
struct Style;

class Window
{
public:

	//specific interface
	Window(const Windows& someWindows, State& aState, const WCHAR* aTitle, const int aX, const int aY, const int aWidth, const int aHeight);
	void frame();
	void text(const int aX, const int aY, const WCHAR* aText, const unsigned int aColor);
	void rect(const int aX, const int aY, const int aWidth, const int aHeight);
	const bool button(const int aX, const int aY, const WCHAR* aText);
	const bool wideButton(const int aX, const int aY, const int aWidth, const WCHAR* aText);
	const bool radio(const bool anActive, const int aX, const int aY, const WCHAR* aText);
	const bool drag(const void* aHandle, const int aX, const int aY, const int aWidth, const int aHeight, const WCHAR* aText);
	const void* drop(const int aX, const int aY, const int aWidth, const int aHeight);
	void hScroll(const float aMax, const float aRange, float& aValue, const int aX, const int aY, const int aWidth, const int aHeight);
	void vScroll(const float aMax, const float aRange, float& aValue, const int aX, const int aY, const int aWidth, const int aHeight);
	const bool node(const void* aHandle, const int aX, const int aY, const WCHAR* aLabel);
	void stringCombo(unsigned int& aChoice, const int aX, const int aY, const WCHAR** someChoices);
	void render(Gdiplus::Graphics& someGraphics);
	const int buttonWidth() const;
	const int buttonHeight() const;
	const POINT clientCursorPos() const;
	const void* dropOutside() const;

	//members
	bool myWantsFocus;

private:

	const bool rect(const Style& aStyle, const int aX, const int aY, const int aWidth, const int aHeight, const bool aSetInside);
	const int fromPercentX(const unsigned char aPercent) const;
	const int fromPercentY(const unsigned char aPercent) const;
	bool& handleState(const void* aHandle);
	const bool isDown(const int aKey) const;
	const bool wasPressed(const int aKey) const;

	struct HandleState
	{
		HandleState()
			:myHandle(NULL)
			,myState(false)
		{
		}

		const void* myHandle;
		bool myState;
	};

	enum
	{
		MAX_TEXTS = 128,
		MAX_RECTS = 128,
		MAX_HANDLES = 128,
	};

	//members
	const Windows& myWindows;
	State& myState;
	const WCHAR* myTitle;
	int myX, myY;
	const int myWidth, myHeight;
	Text myTexts[MAX_TEXTS];
	Rect myRects[MAX_RECTS];
	HandleState myHandles[MAX_HANDLES];
	unsigned int myNumTexts, myNumRects, myNumHandles;
};

}