#include "stdafx.h"
#include "windows.h"

#include "window.h"

namespace GDIView
{

Windows::Windows()
:myNumWindows(0)
{
	unsigned int w;

	for(w = 0; w < MAX_WINDOWS; w++)
		myWindows[w] = NULL;
}

Windows::~Windows()
{
	unsigned int w;

	for(w = 0; w < myNumWindows; w++)
		delete myWindows[w];
}

Window* Windows::window(State& someState, const WCHAR* aTitle, const int aX, const int aY, const int aWidth, const int aHeight)
{
	if(myNumWindows < MAX_WINDOWS)
	{
		myWindows[myNumWindows] = new Window(*this, someState, aTitle, aX, aY, aWidth, aHeight);
		Window* w(myWindows[myNumWindows]);
		myNumWindows++;
		return w;
	}

	return NULL;
}

void Windows::render(const State& someState, Gdiplus::Graphics& someGraphics)
{
	unsigned int w, getsFocus(MAX_WINDOWS);

	//render windows
	for(w = 0; w < myNumWindows; w++)
	{
		if(myWindows[w])
		{
			//check if wants focus
			if(myWindows[w]->myWantsFocus)
			{
				//store this (last one gets it)
				myWindows[w]->myWantsFocus = false;
				getsFocus = w;
			}

			//render
			myWindows[w]->render(someGraphics);
		}
	}

	//check for focus switch
	if(getsFocus < MAX_WINDOWS)
	{
		Window* last(myWindows[myNumWindows - 1]);
		myWindows[myNumWindows - 1] = myWindows[getsFocus];
		myWindows[getsFocus] = last;
	}
}

const bool Windows::hasFocus(const Window& aWindow) const
{
	return &aWindow == myWindows[myNumWindows - 1];
}

}