#include "stdafx.h"
#include "testcontroller.h"

#include "../gdiview/window.h"
#include "../gdiview/view.h"

namespace GuiTest
{

#define NUM_TABS 4
static const WCHAR* TAB_NAMES[NUM_TABS] =
{
	L"Tree Control",
	L"Scroll",
	L"Combo",
	L"Drag n Drop",
};
static const WCHAR* CATEGORY_NAMES[Model::NUM_CATEGORIES] =
{
	L"Fruit",
	L"Veggies",
	L"Meat",
	L"Poultry",
	L"Dairy",
};

//IController
void TestController::doControl(IEventTarget& aTarget)
{
	int x(16), y(64), i;

	myWindow.frame();

	myWindow.text(x, y, L"This is an example of IMGUI using GDI+", -1);
	y += myWindow.buttonHeight();

	for(i = 0; i < NUM_TABS; i++)
	{
		if(myWindow.radio(myTab == i, x, y, TAB_NAMES[i]))
			myTab = i;
		x += myWindow.buttonWidth();
	}

	y += myWindow.buttonHeight();
	switch(myTab)
	{
	case 0:	tree(aTarget, y);	break;
	case 1:	scroll(y);	break;
	case 2:	combo(y);	break;
	case 3:	dragNDrop(aTarget, y);	break;
	}
}

//specific interface
TestController::TestController(const Model& aModel, GDIView::Window& aWindow)
:IController(aModel, aWindow)
,myTab(0)
,myHScroll(0.f)
,myVScroll(0.f)
,myCombo(0)
,mySelection(0)
{
}

void TestController::tree(IEventTarget& aTarget, const int aY)
{
	int x(64), y(aY), c, g;
	GDIView::WString s;

	//traverse model categories
	for(c = 0; c < Model::NUM_CATEGORIES; c++)
	{
		//if categories node is expanded...
		if(myWindow.node(CATEGORY_NAMES[c], x, y += myWindow.buttonHeight(), CATEGORY_NAMES[c]))
		{
			//traverse model goods
			for(g = 0; g < Model::NUM_GOODS; g++)
			{
				//if good belongs to category...
				const Model::Good& good(myModel.myGoods[g]);
				if(good.myCategory == c)
				{
					//if good node is expanded...
					if(myWindow.node(&good, x + 16, y += myWindow.buttonHeight(), good.myName))
					{
						s.format(L"name: %s", good.myName);
						myWindow.text(x + 32, y += myWindow.buttonHeight(), s, -1);

						s.format(L"desc: %s", good.myDescription);
						myWindow.text(x + 32, y += myWindow.buttonHeight(), s, -1);

						myWindow.text(x + 32, y += myWindow.buttonHeight(), L"like it?", -1);
						if(myWindow.radio(good.myLike, x + 96, y, good.myLike ? L"Yes :)" : L"No :("))
							aTarget.toggleGoodLike(g);
					}
				}
			}
		}
	}
}

void TestController::scroll(const int aY)
{
	int x(64), y(aY + 32);

	myWindow.vScroll(100.f, 20.f, myVScroll, x + 256, y, 16, 256);
	myWindow.hScroll(100.f, 80.f, myHScroll, x, y + 256, 256, 16);
}

void TestController::combo(const int aY)
{
	static const WCHAR* COMBO_CHOICES[] =
	{
		L"orange",
		L"pink",
		L"red",
		L"blue",
		L"cyan",
		L"green",
		L"yellow",
		NULL,
	};
	int x(64), y(aY);

	myWindow.stringCombo(myCombo, x, y += myWindow.buttonHeight(), COMBO_CHOICES);
}

void TestController::dragNDrop(IEventTarget& aTarget, const int aY)
{
	const int SLOT(5), INV(64), SPACE(INV), ITEM(48), OFFSET((INV - ITEM) / 2);
	const int X(64), Y(aY + 64);
	const void* d;
	unsigned int i;

	myWindow.text(X, Y - myWindow.buttonHeight() * 4, L"You can drag stuff within the Inventory,", -1);
	myWindow.text(X, Y - myWindow.buttonHeight() * 3, L"and to and from the Ground (OtherController)", -1);

	//drops
	myWindow.text(X, Y - myWindow.buttonHeight(), L"Inventory", -1);
	for(i = 0; i < SLOT; i++)
	{
		d = myWindow.drop(X + i * INV, Y, INV, INV);
		if(d)
			aTarget.addToInventory(itemIndex(d), i);
	}

	//drags
	for(i = 0; i < Model::NUM_ITEMS; i++)
	{
		const Model::Item* item(myModel.myItems + i);
		if(item->myInventory)
			myWindow.drag(item, item->mySlot * SPACE + X + OFFSET, Y + OFFSET, ITEM, ITEM, ITEM_NAMES[i]);
	}
}

}